# All rights reserved. Written by Gilles Lenfant <glenfant@bigfoot.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__doc__ = """Widgets / methods for ordering properties in a PropertyManager

$Id: Patch.py,v 1.5 2003/06/11 08:16:18 gilles Exp $"""
__version__ = '0.1.0'

from OFS.PropertyManager import PropertyManager
from Globals import DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import manage_properties
from zLOG import LOG, INFO

PropertyManager.security = ClassSecurityInfo()

# *****************************************************************
# ** Plugs hereafter methods to replace/add PropertyManager ones **
# *****************************************************************

manage_propertiesForm=DTMLFile('zmi/properties', globals(), property_extensible_schema__=1)
PropertyManager.manage_propertiesForm = manage_propertiesForm

def manage_moveDown(self, property_id, REQUEST=None):
    """Moves a property down in the proprerty sheet"""
    # XXX FIXME : trap bad property_id and meaningful new position
    newprops = []
    i = 0
    for p in self._properties:
        if p['id'] == property_id:
            position = i
            moved = p
        else:
            newprops.append(p)
            i += 1
    newprops.insert(position + 1, moved)
    self._properties = tuple(newprops)
    if REQUEST is not None:
        message = "Property %s moved down" % property_id
        return self.manage_propertiesForm(self,REQUEST, manage_tabs_message=message)
    return
# XXX FIXME: This does not work (Anonymous can use 'manage_moveDown')
PropertyManager.security.declareProtected(manage_properties, 'manage_moveDown')
PropertyManager.manage_moveDown = manage_moveDown

def manage_moveUp(self, property_id, REQUEST=None):
    """Moves a property up in the proprerty sheet"""
    # XXX FIXME : trap bad property_id and meaningful new position
    newprops = []
    for p in self._properties:
        if p['id'] == property_id:
            newprops.insert(len(newprops) - 1, p)
        else:
            newprops.append(p)
    self._properties = tuple(newprops)
    if REQUEST is not None:
        message = "Property %s moved up" % property_id
        return self.manage_propertiesForm(self,REQUEST, manage_tabs_message=message)
    return
PropertyManager.security.declareProtected(manage_properties, 'manage_moveUp')
PropertyManager.manage_moveUp = manage_moveUp

def manage_moveTop(self, property_id, REQUEST=None):
    """Moves a property to the top of a property sheet"""
    # XXX FIXME : trap bad property_id and meaningful new position
    newprops = []
    for p in self._properties:
        if p['id'] == property_id:
            moved = p
        else:
            newprops.append(p)
    newprops.insert(0, moved)
    self._properties = tuple(newprops)
    if REQUEST is not None:
        message = "Property %s moved to the top" % property_id
        return self.manage_propertiesForm(self,REQUEST, manage_tabs_message=message)
    return
PropertyManager.security.declareProtected(manage_properties, 'manage_moveTop')
PropertyManager.manage_moveTop = manage_moveTop

def manage_moveEnd(self, property_id, REQUEST=None):
    """Moves a property to the end of the property sheet"""
    # XXX FIXME : trap bad property_id and meaningful new position
    newprops = []
    for p in self._properties:
        if p['id'] == property_id:
            moved = p
        else:
            newprops.append(p)
    newprops.append(moved)
    self._properties = tuple(newprops)
    if REQUEST is not None:
        message = "Property %s moved to the end" % property_id
        return self.manage_propertiesForm(self,REQUEST, manage_tabs_message=message)
    return
PropertyManager.security.declareProtected(manage_properties, 'manage_moveEnd')
PropertyManager.manage_moveEnd = manage_moveEnd

LOG('OrderablePropertyManager', INFO, 'Applied patch version %s.' % __version__)

InitializeClass(PropertyManager)